#!/usr/bin/env bash

function errormsg () {
	echo "ERROR: setup was not successfull!" 1>&2
	echo "Carefully inspect any error messages printed before." 1>&2
	echo "Unable to continue, exiting..." 1>&2
	exit 1
}

function checkfiles () {
	if [ -f /etc/wpa_supplicant/citec-root-ca.crt ] ||
		[ -f /etc/wpa_supplicant/dot1x-wired.crt ] ||
		[ -f /etc/wpa_supplicant/dot1x-wired.sh ] ||
		[ -f /etc/wpa_supplicant/dot1x-wired.conf ]; then

		echo "Warning: some files already exist:"
		echo "------------------------------------------------------"
		ls -la /etc/wpa_supplicant/citec-root-ca.crt \
			/etc/wpa_supplicant/dot1x-wired.crt \
			/etc/wpa_supplicant/dot1x-wired.sh \
			/etc/wpa_supplicant/dot1x-wired.conf
		echo "------------------------------------------------------"
		echo ""

		local choice=
		echo -n "Overwrite these files (y/N)? "
		read choice
		echo ""

		if [ "${choice}" = "Y" ] || [ "${choice}" = "y" ]; then
			rm -f /etc/wpa_supplicant/citec-root-ca.crt || errormsg
			rm -f /etc/wpa_supplicant/dot1x-wired.crt || errormsg
			rm -f /etc/wpa_supplicant/dot1x-wired.sh || errormsg
			rm -f /etc/wpa_supplicant/dot1x-wired.conf || errormsg
		else
			echo "Not overwriting existing files." 1>&2
			echo "Unable to continue, exiting..." 1>&2
			exit 1
		fi
	fi
}

function checkdebian () {
	if [ ! -f /etc/debian_version ]; then
		echo "ERROR: This is not a Debian based system!" 1>&2
		echo "Unable to continue, exiting..." 1>&2
		exit 1
	fi
}

function checkroot () {
	if [ "$( whoami )" != "root" ]; then
		echo "ERROR: This script must be run as root!" 1>&2
		echo "Unable to continue, exiting..." 1>&2
		exit 1
	fi
}

function writerootca () {
	echo -n "Saving IEEE-802.1x root CA certificate to /etc/wpa_supplicant/citec-root-ca.crt... "
	mkdir -p /etc/wpa_supplicant
	cat - <<EOF > /etc/wpa_supplicant/citec-root-ca.crt
-----BEGIN CERTIFICATE-----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-----END CERTIFICATE-----
EOF
	echo "Done."

	echo -n "Creating link to /etc/wpa_supplicant/dot1x-wired.crt... "
	ln -s /etc/wpa_supplicant/citec-root-ca.crt /etc/wpa_supplicant/dot1x-wired.crt
	echo "Done."
}

function installwpa () {
	which wpa_supplicant &> /dev/null
	if [ ${?} -ne 0 ]; then
		echo "wpa_supplicant is not installed."
		echo -n "Do you want to try to install (Y/n)? "
		local choice=
		read choice
		echo ""

		if [ "${choice}" = "Y" ] || [ "${choice}" = "Y" ] || [ "${choice}" = "" ]; then
			echo "----------------------------------------------------------------------------"
			echo "Installing wpa_supplicant..."
			echo "----------------------------------------------------------------------------"
			apt-get update
			apt-get install --yes wpasupplicant || (
				echo "ERROR: Unable to install wpa_supplicant!" 1>&2
				echo "Install manually (apt-get install wpasupplicant) and rerun this script." 1>&2
				echo "Exiting..." 1>&2
				exit 1
			)
			echo "----------------------------------------------------------------------------"
			echo "Installation was successfull!"
			echo "----------------------------------------------------------------------------"
			echo ""
		else
			echo "ERROR: Unable to continue without wpa_supplicant!" 1>&2
			echo "Install manually (apt-get install wpasupplicant) and rerun this script." 1>&2
			echo "Exiting..." 1>&2
			exit 1
		fi
	fi
}

function createconf () {
	echo -n "Creating wpa_supplicant configuration file /etc/wpa_supplicant/dot1x-wired.conf... "
	cat - <<EOF > /etc/wpa_supplicant/dot1x-wired.conf
#
# /etc/wpa_supplicant/dot1x-wired.conf
# wpa_supplicant configuration for wired port based authentication (IEEE-802.1x)
#

ctrl_interface=/var/run/dot1x-wired.ctrl
ctrl_interface_group=0
eapol_version=2
ap_scan=0

network={
	key_mgmt=IEEE8021X
	eap=TTLS
	phase2="auth=PAP"
	eapol_flags=0
	ca_cert="/etc/wpa_supplicant/dot1x-wired.crt"
	identity="${identity}"
	password="${password}"
}
EOF

	chmod 0600 /etc/wpa_supplicant/dot1x-wired.conf
	chown root:root /etc/wpa_supplicant/dot1x-wired.conf
	echo "Done."
}

function createscript () {
	echo -n "Saving startup script to /etc/wpa_supplicant/dot1x-wired.sh... "
	cat - <<EOF > /etc/wpa_supplicant/dot1x-wired.sh
#!/bin/bash

action="\${1}"
interface="\${2}"

test -z "\${action}" && exit 1
test -z "\${interface}" && exit 1

case "\${action}" in
        "start")
		wpa_supplicant -c/etc/wpa_supplicant/dot1x-wired.conf -t -f/var/log/dot1x-wired.log -i\${interface} -Dwired -B -P/var/run/dot1x-wired.pid
		sleep 1
		for counter in \$( seq 1 30 ); do
			if [ ! -z "\$( wpa_cli -p/var/run/dot1x-wired.ctrl status | grep "suppPortStatus=Authorized" )" ]; then
				break
			fi
			sleep 1
		done
		;;
	"stop")
		kill -TERM \$( cat /var/run/dot1x-wired.pid 2> /dev/null ) 2> /dev/null
		;;
	*)
		exit 1
		;;
esac

exit 0
EOF

chmod +x /etc/wpa_supplicant/dot1x-wired.sh
echo "Done."
}

function checkinterfaces () {
	echo -n "Looking for network interfaces in /etc/network/interfaces... "

	local intcount="$( grep "^\s*iface\s*.*\s*inet\s*dhcp" /etc/network/interfaces | wc -l  )"
	if [ ${intcount} -eq 1 ]; then
		local int="$( grep -e "^\s*iface\s*.*\s*inet\s*dhcp" /etc/network/interfaces | awk '{ print $2 }' )"
		echo "Done."
		echo "Found one interface: ${int}"
		echo  -n "Enable IEEE-802.1x for interface ${int} (Y/n)?"
		local choice=
		read choice
		echo ""

		if [ "${choice}" = "Y" ] || [ "${choice}" = "Y" ] || [ "${choice}" = "" ]; then
			interface="${int}"
		else
			echo "Exiting..."
			exit 1
		fi
	elif [ ${intcount} -lt 1 ]; then
		cat - <<EOF
More than one interfaces configuraured or no interface configuration found.
Please edit /etc/network/interfaces manually.
A configuration for eth0 should look like this:

auto eth0
iface eth0 inet dhcp
	pre-up /etc/wpa_supplicant/dot1x-wired.sh start \$IFACE
	post-down /etc/wpa_supplicant/dot1x-wired.sh stop \$IFACE

Please add pre-up and post-down lines manually to get IEEE-802.1x
authentication working!

EOF
		interface="none"
	fi
}

function modifyinterfaces () {
	if [ "${interface}" != "none" ] && [ "${interface}" != "" ]; then
		echo -n "Now adding startup commands to /etc/network/interfaces... "
		cp -p /etc/network/interfaces /etc/network/interfaces.orig
		cat /etc/network/interfaces.orig | grep -v "/etc/wpa_supplicant/dot1x-wired.sh" > /etc/network/interfaces
		sed -i "s/^\s*iface\s*.*\s*inet\s*dhcp/iface ${interface} inet dhcp\n\tpre-up \/etc\/wpa_supplicant\/dot1x-wired.sh start \$IFACE\n\tpost-down \/etc\/wpa_supplicant\/dot1x-wired.sh stop \$IFACE/g" /etc/network/interfaces
		echo "Done."
		echo "Saved original interfaces to /etc/network/interfaces.orig"
	fi
}

function main () {
	cd /tmp

	checkdebian
	checkroot
	installwpa

	echo "+----------------------------------------------------------+"
	echo "| Setup IEEE-802.1x wired authentication                   |"
	echo "|                                                          |"
	echo "| For wired network access at TechFak, CITEC and CoR-Lab   |"
	echo "+----------------------------------------------------------+"
	echo ""
	echo "Press <ENTER> to continue, <CTRL>-<C> to abort."
	read key

	identity_default="mach-$( hostname -s )"
	echo -n "Please enter IEEE-802.1x identity (Press enter for default \"${identity_default}\"): "
	read identity
	if [ -z "${identity}" ]; then
		identity="${identity_default}"
	fi

	password=
	echo -n "Please enter password for \"${identity}\": "
	read password
	if [ -z "${password}" ]; then
		echo "ERROR: empty password not allowed!" 1>&2
		echo "Aborting!" 1>&2
		exit 1
	fi

	echo ""
	echo "================================================"
	echo "Summary:"
	echo ""
	echo "IEEE-802.1x   identity:    ${identity}"
	echo "              password:    ${password}"
	echo "================================================"
	echo ""
	choice=
	echo -n "Continue (Y/n)? "
	read choice
	if [ ! -z "${choice}" ] && [ "${choice}" != "y" ] && [ "${choice}" != "Y" ]; then
		echo ""
		echo "Aborting."
		exit 0
	fi

	echo ""

	checkinterfaces
	checkfiles
	writerootca
	createconf
	createscript
	modifyinterfaces

	echo "+----------------------------------------------------------+"
	echo "| IEEE-802.1x setup is now complete.                       |"
	echo "|                                                          |"
	echo "| You need to restart networking or your system...         |"
	echo "+----------------------------------------------------------+"
}

main
exit 0
